# Vportage.py
# Author: Ryan Mulligan
# Version: .2
#
# The purpose of this is to move vpython programs into szysgy
#
#
# At all points where I have dsg I mean distributed scene graph
#
# TODO: Most Primatives are not supported. 
#       Scene Events are not supported
################################################################################
################################################################################

#import szyszgy
from PySZG import *
import time
import random
#we import this so we can call super methods properly in the primatives
import gvportage

###Magical Numbers and Constant Declarations
################################################################################
################################################################################
vpython_to_dsg_scale_factor = 1/(2.1)

#some aliases to make my code more readable
translation_matrix = ar_TM
scale_matrix = ar_SM
rotation_matrix =  ar_RM


#####Utilities
################################################################################
################################################################################

class Vector3:
	def __init__(self, parent_frame, x=0,y=0,z=0):
		self.__x = x
		self.__y = y
		self.__z = z
		self.__parent_frame = parent_frame
	
	def get_x(self):
		return self.__x
	def set_x(self,ax):
		print "setting x"
		self.__parent_frame.update_pos()
		self.__x = ax
	
	x = property(get_x, set_x)
	
	def get_y(self,):
		return self.__y
	def set_y(self,ay):
		print "setting y"
		self.__parent_frame.update_pos()
		self.__y = ay+10
	
	y = property(get_y, set_y)
	
	def get_z(self):
		return self.__z
	def set_z(self,az):
		print "setting z"
		self.__parent_frame.update_pos()
		self.__z = az
	
	z = property(get_z, set_z)
	
	
class scene:
	pass
	

	
	
#Adds lights to an arbitrary scenegraph node. Probably should be the root.
def add_lights_to(g):
	r = g.getRoot()
	l = r.new("light")
	# Create a new light description. This will be used to configure the scene graph node.
	light = arLight()
	# It is important to give each light a unique ID (0-7).
	light.lightID = 0
	# The fourth position being 0 means this is a directional light (as in OpenGL).
	# If it were 1, this would be a positional light.
	light.position = arVector4(0,0,-1,0)
	# Non-directional component of the light (with RGB color components), as in OpenGL.
        light.ambient = arVector3(.3,.3,.3)
	# The directional component of the light (with RGB color components), as in OpenGL.
        light.diffuse = arVector3(0.5, 0.5, 0.5)
	# The scene graph node is now configured with the light description.
	l.setLight(light)
	l = r.new("light")
	light = arLight()
	light.lightID = 1
	light.position = arVector4(0,0,1,0)
        light.ambient = arVector3(.3,.3,.3)
        light.diffuse = arVector3(0.5, 0.5, 0.5)
	l.setLight(light)
	
class color:
	black = (0,0,0)
	white = (1,1,1)
	
	red = (1,0,0)
	green = (0,1,0)
	blue = (0,0,1)
	
	yellow = (1,1,0)
	cyan = (0,1,1)
	magenta = (1,0,1)
	
	orange = (1,0.6,0)
	
#this goes against my best morals but I'm trying to be compatible with visual
false = False
true = True

from math import pi
pi = pi





###Primatives
################################################################################
################################################################################
class frame(object):
	def __init__(self, transform_node =None, frame =None, pos = (0,0,0)):
		parent_frame = frame
		position_offset_from_parent = pos
		
		self.__pos = Vector3(self,0,0,0)
		self.__translation_matrix = translation_matrix(self.__pos.x,self.__pos.y,self.__pos.z)
		self.__rotation_matrix = rotation_matrix('z',0)
		self.__size = Vector3(self,1,1,1)
		self.__scale_matrix = scale_matrix(self.__size.x,self.__size.y,self.__size.z)
		
		try:
			if parent_frame:
				self.node = parent_frame.node.new("transform")
			else:
				self.node = transform_node.new("transform")
		except:
			raise Exception, "You cannot use frame without a node to attach to in Geometry only mode. Please pass frame a transform_node or another frame"
			
		self.pos = position_offset_from_parent
	
	def set_pos(self, position_offset_from_parent):
		self.__pos = Vector3(self,position_offset_from_parent[0],position_offset_from_parent[1],position_offset_from_parent[2])
		self.__translation_matrix = translation_matrix(self.__pos.x,self.__pos.y,self.__pos.z)
		
		self.update_transform()
	
	def update_pos(self):
		print "updating pos"
		self.__translation_matrix = translation_matrix(self.__pos.x,self.__pos.y,self.__pos.z)
		self.update_transform()
		
	def get_pos(self):
		return self.__pos
		
	#this makes pos work like pos in vpython
	#so you can call frame.pos = (0,0,1) and it works.
	pos = property(get_pos, set_pos)
	
	def update_transform(self):
		self.node.setTransform(self.__translation_matrix*self.__scale_matrix*self.__rotation_matrix)
		
	def rotate(self, angle, axis):
		if (axis == (0,0,1)):
			axis_character = 'z'
		elif (axis == (0,1,0)):
			axis_character = 'y'
		elif (axis == (1,0,0)):
			axis_character = 'x'
		else:
			axis_character = None
		
		
		if axis_character:
			self.__rotation_matrix = self.__rotation_matrix * rotation_matrix(axis_character, angle)
		else:
			self.__rotation_matrix = self.__rotation_matrix * rotation_matrix(axis, angle)
			
		self.update_transform()
	
	def resize(self, x,y,z):
		self.__size = Vector3(self,x*vpython_to_dsg_scale_factor,y*vpython_to_dsg_scale_factor,z*vpython_to_dsg_scale_factor)
		self.__scale_matrix = scale_matrix(self.__size.x,self.__size.y,self.__size.z)
		self.update_transform()
	
	def get_size(self):
		return self.__size
	def set_size(self, xyz_scale):
	  self.resize(*xyz_scale)
		
	size = property(get_size,set_size)
	


class sphere(frame):
	def __init__(self,frame, pos = (0,0,0), color = (0,0,0), radius = 1):
		parent_frame = frame
		position_offset_from_parent = pos
		rgb_color = color
		
		gvportage.frame.__init__(self, transform_node =None, frame = parent_frame, pos = position_offset_from_parent)
		
		self.radius = radius
		self.material_node = self.node.new("material")
		
		self.material = arMaterial()
		self.material.diffuse = arVector3(*rgb_color)
		
		self.material_node.set(self.material)
		
		sphere_mesh = arSphereMesh()
		sphere_mesh.setAttributes(15)
		sphere_mesh.attachMesh(self.material_node)
		
	def get_radius(self):
		return self.__size.x
	def set_radius(self, radius):
	  self.resize(radius,radius,radius)
	radius = property(get_radius,set_radius)		

class ellipsoid(sphere):
	def __init__(self,frame, pos=(0,0,0), color=(0,0,0), size = (1,1,1), length = 1, width = 1, depth = 1):
		parent_frame = frame
		position_offset_from_parent = pos
		rgb_color = color
		
		gvportage.frame.__init__(self,transform_node =None,frame = parent_frame, pos = position_offset_from_parent)
		
		self.size = size
		
		if length != 1 or width != 1 or height != 1:
			self.size = (length,depth,width)
		
		self.material_node = self.node.new("material")
		
		self.material = arMaterial()
		self.material.diffuse = arVector3(*rgb_color)
		
		self.material_node.set(self.material)
		
		sphere_mesh = arSphereMesh()
		sphere_mesh.setAttributes(15)
		sphere_mesh.attachMesh(self.material_node)
		
class box(frame):
	def __init__(self,frame, pos = (0,0,0), color = (0,0,0), size = (1,1,1), length = 1, height = 1, width = 1):
		parent_frame = frame
		position_offset_from_parent = pos
		rgb_color = color
		
		gvportage.frame.__init__(self,transform_node =None,frame = parent_frame, pos = position_offset_from_parent)
		
		self.size = size
		if length != 1 or width != 1 or height != 1:
			self.size = (length,height,width)
		
		self.material_node = self.node.new("material")
		
		self.material = arMaterial()
		self.material.diffuse = arVector3(*rgb_color)
		
		self.material_node.set(self.material)
		
		sphere_mesh =  arCubeMesh()
		sphere_mesh.attachMesh(self.material_node)
